@@ -2,6 +2,7 @@ __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
 import inspect
+import typing
 from functools import wraps
 from typing import (
     Any,
@@ -11,15 +12,23 @@ from typing import (
     Optional,
     Sequence,
     Iterable,
+    List,
+    Union,
 )
 
+import numpy as np
 from google.protobuf.struct_pb2 import Struct
 
 from ..enums import OnErrorStrategy
-from ..executors import BaseExecutor
+from ..excepts import LengthMismatchException
 from ..executors.compound import CompoundExecutor
 from ..executors.decorators import wrap_func
-from ..helper import convert_tuple_to_list
+from ..helper import (
+    convert_tuple_to_list,
+    cached_property,
+    find_request_binding,
+    _canonical_request_name,
+)
 from ..jaml import JAMLCompatible
 from ..types.querylang import QueryLang
 from ..types.sets import DocumentSet
@@ -153,8 +162,8 @@ class DriverType(type(JAMLCompatible), type):
     def __new__(cls, *args, **kwargs):
         """Create and register a new class with this meta class.
 
-        :param *args: *args for super
-        :param **kwargs: **kwargs for super
+        :param args: additional positional arguments which are just used for the parent initialization
+        :param kwargs: additional key value arguments which are just used for the parent initialization
         :return: the newly registered class
         """
         _cls = super().__new__(cls, *args, **kwargs)
@@ -187,8 +196,8 @@ class BaseDriver(JAMLCompatible, metaclass=DriverType):
 
     :param priority: the priority of its default arg values (hardcoded in Python). If the
          received ``QueryLang`` has a higher priority, it will override the hardcoded value
-    :param *args: not used (kept to maintain interface)
-    :param **kwargs: not used (kept to maintain interface)
+    :param args: not used (kept to maintain interface)
+    :param kwargs: not used (kept to maintain interface)
     """
 
     store_args_kwargs = False  #: set this to ``True`` to save ``args`` (in a list) and ``kwargs`` (in a map) in YAML config
@@ -203,8 +212,8 @@ class BaseDriver(JAMLCompatible, metaclass=DriverType):
         """Attach this driver to a :class:`jina.peapods.runtimes.zmq.zed.ZEDRuntime`
 
         :param runtime: the pea to be attached
-        :param *args: not used (kept to maintain interface)
-        :param **kwargs: not used (kept to maintain interface)
+        :param args: not used (kept to maintain interface)
+        :param kwargs: not used (kept to maintain interface)
         """
         self.runtime = runtime
         self.attached = True
@@ -322,7 +331,7 @@ class BaseDriver(JAMLCompatible, metaclass=DriverType):
 
 class ContextAwareRecursiveMixin:
     """
-    The full datastructure version of :class:`FlatRecursiveMixin`, to be mixed in with :class:`BaseRecursiveDriver`.
+    The full data structure version of :class:`FlatRecursiveMixin`, to be mixed in with :class:`BaseRecursiveDriver`.
     It uses :meth:`traverse` in :class:`DocumentSet` and allows direct manipulation of Chunk-/Match-/DocumentSets.
 
     .. seealso::
@@ -333,8 +342,8 @@ class ContextAwareRecursiveMixin:
     def __call__(self, *args, **kwargs):
         """Traverse with _apply_all
 
-        :param *args: *args forwarded to ``_apply_all``
-        :param **kwargs: **kwargs forwarded to ``_apply_all``
+        :param args: args forwarded to ``_apply_all``
+        :param kwargs: kwargs forwarded to ``_apply_all``
         """
         document_sets = self.docs.traverse(self._traversal_paths)
         self._apply_all(document_sets, *args, **kwargs)
@@ -353,8 +362,8 @@ class ContextAwareRecursiveMixin:
         Adding a chunk to a ChunkSet results in adding a chunk to the parent Document.
 
         :param doc_sequences: the Documents that should be handled
-        :param *args: driver specific arguments, which might be forwarded to the Executor
-        :param **kwargs: driver specific arguments, which might be forwarded to the Executor
+        :param args: driver specific arguments, which might be forwarded to the Executor
+        :param kwargs: driver specific arguments, which might be forwarded to the Executor
         """
 
 
@@ -372,8 +381,8 @@ class FlatRecursiveMixin:
     def __call__(self, *args, **kwargs):
         """Traverse with _apply_all
 
-        :param *args: *args forwarded to ``_apply_all``
-        :param **kwargs: **kwargs forwarded to ``_apply_all``
+        :param args: args forwarded to ``_apply_all``
+        :param kwargs: kwargs forwarded to ``_apply_all``
         """
         path_documents = self.docs.traverse_flattened_per_path(self._traversal_paths)
         for documents in path_documents:
@@ -388,15 +397,200 @@ class FlatRecursiveMixin:
     ) -> None:
         """Apply function works on a list of docs, modify the docs in-place.
 
-        The list refers to all reachable leafes of a single ``traversal_path``.
+        The list refers to all reachable leaves of a single ``traversal_path``.
 
         :param docs: the Documents that should be handled
-        :param *args: driver specific arguments, which might be forwarded to the Executor
-        :param **kwargs: driver specific arguments, which might be forwarded to the Executor
+        :param args: driver specific arguments, which might be forwarded to the Executor
+        :param kwargs: driver specific arguments, which might be forwarded to the Executor
 
         """
 
 
+class DocsExtractUpdateMixin:
+    """
+    A Driver pattern for extracting attributes from Documents, feeding to an executor and updating the Documents with
+    the results.
+
+    Drivers equipped with this mixin will have :method:`_apply_all` inherited.
+
+    The :method:`_apply_all` implements the following logics:
+        - From ``docs``, it extracts the attributes defined :method:`exec_fn`'s arguments.
+        - It feeds the attributes to the bind executor's :method:`exec_fn`.
+        - It updates ``docs`` with results returned from :method:`exec_fn`
+
+    The following shortcut logics are implemented:
+        - while extracting: attributes defined :method:`exec_fn`'s arguments are extracted from ``docs``;
+        - while extracting: attributes annotated with ``ndarray`` are stacked into Numpy NdArray objects;
+        - while updating: if ``exec_fn`` returns a List of Dict, then ``doc.set_attrs(**exec_result)`` is called;
+        - while updating: if ``exec_fn`` returns a Document, then ``doc.update(exec_result)` is called.
+        - while updating: if none of above applies, then calling :meth:`update_single_doc`
+
+    To override the update behavior, you can choose to override:
+        - :meth:`update_docs` if you want to modify the behavior of updating docs in bulk
+        - :meth:`update_single_doc` if you want to modify the behavior of updating a single doc
+    """
+
+    def _apply_all(self, docs: 'DocumentSet') -> None:
+        """Apply function works on a list of docs, modify the docs in-place.
+
+        The list refers to all reachable leaves of a single ``traversal_path``.
+
+        :param docs: the Documents that should be handled
+        """
+
+        contents, docs_pts = docs.extract_docs(
+            *self._exec_fn_required_keys,
+            stack_contents=self._exec_fn_required_keys_is_ndarray,
+        )
+
+        if docs_pts:
+            if len(self._exec_fn_required_keys) > 1:
+                exec_results = self.exec_fn(*contents)
+            else:
+                exec_results = self.exec_fn(contents)
+
+            if exec_results is not None:
+                # if exec_fn returns None then exec_fn is assumed to be immutable wrt. doc, hence skipped
+
+                try:
+                    len_results = len(exec_results)
+                except:
+                    try:
+                        len_results = exec_results.shape[0]
+                    except:
+                        len_results = None
+
+                if len(docs_pts) != len_results:
+                    msg = (
+                        f'mismatched {len(docs_pts)} docs from level {docs_pts[0].granularity} '
+                        f'and length of returned: {len_results}, their length must be the same'
+                    )
+                    raise LengthMismatchException(msg)
+
+                self.update_docs(docs_pts, exec_results)
+
+    def update_docs(
+        self,
+        docs_pts: 'DocumentSet',
+        exec_results: Union[List[Dict], List['Document'], Any],
+    ) -> None:
+        """
+        Update Documents with the Executor returned results.
+
+        :param: docs_pts: the set of document to be updated
+        :param: exec_results: the results from :meth:`exec_fn`
+        """
+        from ..types.document import Document
+
+        if self._exec_fn_return_is_ndarray and not isinstance(exec_results, np.ndarray):
+            r_type = type(exec_results).__name__
+            if r_type in {'EagerTensor', 'Tensor', 'list'}:
+                exec_results = np.array(exec_results, dtype=np.float32)
+            else:
+                raise TypeError(f'unrecognized type {exec_results!r}')
+
+        for doc, exec_result in zip(docs_pts, exec_results):
+            if isinstance(exec_result, dict):
+                doc.set_attrs(**exec_result)
+            elif isinstance(exec_result, Document):
+                # doc id should not be override with this method
+                doc.update(exec_result, exclude_fields=('id',))
+            else:
+                self.update_single_doc(doc, exec_result)
+
+    def update_single_doc(self, doc: 'Document', exec_result: Any) -> None:
+        """Update a single Document with the Executor returned result.
+
+        :param doc: the Document object
+        :param exec_result: the single result from :meth:`exec_fn`
+        """
+        raise NotImplementedError
+
+    @cached_property
+    def _exec_fn_required_keys(self) -> List[str]:
+        """Get the arguments of :attr:`exec_fn`.
+
+        If ``strict_method_args`` set, then all arguments of :attr:`exec_fn` must be valid :class:`Document` attribute.
+
+        :return: a list of supported arguments
+        """
+
+        if not self.exec_fn:
+            raise ValueError(
+                f'`exec_fn` is None, maybe {self} is not attached? call `self.attach`.'
+            )
+
+        required_keys = [
+            k
+            for k in inspect.getfullargspec(inspect.unwrap(self.exec_fn)).args
+            if k != 'self'
+        ]
+        if not required_keys:
+            raise AttributeError(f'{self.exec_fn} takes no argument.')
+
+        if self._strict_method_args:
+            from ..proto import jina_pb2
+            from .. import Document
+
+            support_keys = Document.get_all_attributes()
+            unrecognized_keys = set(required_keys).difference(support_keys)
+            if unrecognized_keys:
+                camel_keys = set(
+                    jina_pb2.DocumentProto().DESCRIPTOR.fields_by_camelcase_name
+                )
+                legacy_keys = {'data'}
+                unrecognized_camel_keys = unrecognized_keys.intersection(camel_keys)
+                if unrecognized_camel_keys:
+                    raise AttributeError(
+                        f'{unrecognized_camel_keys} are supported but you give them in CamelCase, '
+                        f'please rewrite them in canonical form.'
+                    )
+                elif unrecognized_keys.intersection(legacy_keys):
+                    raise AttributeError(
+                        f'{unrecognized_keys.intersection(legacy_keys)} is now deprecated and not a valid argument of '
+                        'the executor function, '
+                        'please change `data` to `content: \'np.ndarray\'` in your executor function. '
+                        'details: https://github.com/jina-ai/jina/pull/2313/'
+                    )
+                else:
+                    raise AttributeError(
+                        f'{unrecognized_keys} are invalid Document attributes, must come from {support_keys}'
+                    )
+
+        return required_keys
+
+    @cached_property
+    def _exec_fn_required_keys_is_ndarray(self) -> List[bool]:
+        """Return a list of boolean indicators for showing if a key is annotated as ndarray
+
+        :return: a list of boolean idicator, True if the corresponding key is annotated as ndarray
+        """
+
+        try:
+            anno = typing.get_type_hints((inspect.unwrap(self.exec_fn)))
+            return [
+                anno.get(k, None) == np.ndarray for k in self._exec_fn_required_keys
+            ]
+        except NameError:
+            return [False] * len(self._exec_fn_required_keys)
+
+    @cached_property
+    def _exec_fn_return_is_ndarray(self) -> bool:
+        """Return a boolean value for showing if the return of :meth:`exec_fn` is annotated as `ndarray`
+
+        :return: a bool indicator
+        """
+        try:
+            return (
+                typing.get_type_hints((inspect.unwrap(self.exec_fn))).get(
+                    'return', None
+                )
+                == np.ndarray
+            )
+        except NameError:
+            return False
+
+
 class BaseRecursiveDriver(BaseDriver):
     """A :class:`BaseRecursiveDriver` is an abstract Driver class containing information about the `traversal_paths`
     that a `Driver` must apply its logic.
@@ -407,8 +601,8 @@ class BaseRecursiveDriver(BaseDriver):
         """Initialize a :class:`BaseRecursiveDriver`
 
         :param traversal_paths: Describes the leaves of the document tree on which _apply_all are called
-        :param *args: *args for super
-        :param **kwargs: **kwargs for super
+        :param args: additional positional arguments which are just used for the parent initialization
+        :param kwargs: additional key value arguments which are just used for the parent initialization
         """
         super().__init__(*args, **kwargs)
         self._traversal_paths = [path.lower() for path in traversal_paths]
@@ -427,6 +621,7 @@ class BaseExecutableDriver(BaseRecursiveDriver):
         self,
         executor: Optional[str] = None,
         method: Optional[str] = None,
+        strict_method_args: bool = True,
         *args,
         **kwargs,
     ):
@@ -434,12 +629,14 @@ class BaseExecutableDriver(BaseRecursiveDriver):
 
         :param executor: the name of the sub-executor, only necessary when :class:`jina.executors.compound.CompoundExecutor` is used
         :param method: the function name of the executor that the driver feeds to
-        :param *args: *args for super
-        :param **kwargs: **kwargs for super
+        :param strict_method_args: if set, then the input args of ``executor.method`` must be valid :class:`Document` attributes
+        :param args: additional positional arguments which are just used for the parent initialization
+        :param kwargs: additional key value arguments which are just used for the parent initialization
         """
         super().__init__(*args, **kwargs)
         self._executor_name = executor
         self._method_name = method
+        self._strict_method_args = strict_method_args
         self._exec = None
         self._exec_fn = None
 
@@ -458,7 +655,9 @@ class BaseExecutableDriver(BaseRecursiveDriver):
 
         :return: the Callable to execute in the driver
         """
-        if (
+        if not self.runtime:
+            return self._exec_fn
+        elif (
             not self.msg.is_error
             or self.runtime.args.on_error_strategy < OnErrorStrategy.SKIP_EXECUTOR
         ):
@@ -466,12 +665,15 @@ class BaseExecutableDriver(BaseRecursiveDriver):
         else:
             return lambda *args, **kwargs: None
 
-    def attach(self, executor: 'AnyExecutor', *args, **kwargs) -> None:
+    def attach(
+        self, executor: 'AnyExecutor', req_type: Optional[str] = None, *args, **kwargs
+    ) -> None:
         """Attach the driver to a :class:`jina.executors.BaseExecutor`
 
         :param executor: the executor to which we attach
-        :param *args: *args for super().attach()
-        :param **kwargs: **kwargs for super().attach()
+        :param req_type: the request type to attach to
+        :param args: additional positional arguments for the call of super().attach()
+        :param kwargs: additional key value arguments for the call of super().attach()
         """
         super().attach(*args, **kwargs)
         if self._executor_name and isinstance(executor, CompoundExecutor):
@@ -492,13 +694,16 @@ class BaseExecutableDriver(BaseRecursiveDriver):
         else:
             self._exec = executor
 
-        if self._method_name:
-            if self._method_name not in BaseExecutor.exec_methods:
-                self.logger.warning(
-                    f'Using method {self._method_name} as driver execution function which is not registered'
-                    f'as a potential `exec_method` of an Executor. It won\'t work if used inside a CompoundExecutor'
-                )
+        if not self._method_name:
+            decor_bindings = find_request_binding(self.exec.__class__)
+            if req_type:
+                canonic_name = _canonical_request_name(req_type)
+            if req_type and canonic_name in decor_bindings:
+                self._method_name = decor_bindings[canonic_name]
+            elif 'default' in decor_bindings:
+                self._method_name = decor_bindings['default']
 
+        if self._method_name:
             self._exec_fn = getattr(self.exec, self._method_name)
 
     def __getstate__(self) -> Dict[str, Any]:
