@@ -1,4 +1,4 @@
-__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
+__copyright__ = "Copyright (c) 2021 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
 import os
@@ -15,22 +15,28 @@ from jina.excepts import PretrainedModelFileDoesNotExist
 
 class CompressionVaeEncoder(TFDevice, BaseNumericEncoder):
     """
-    :class:`CompressionVaeEncoder` is a dimensionality reduction tool based on the idea of
-    Variational Autoencoders. It encodes data from an ndarray in size `B x T` into an ndarray in size `B x D`.
+    :class:`CompressionVaeEncoder` is a dimensionality reduction tool.
 
-    Full code and documentation can be found here: https://github.com/maxfrenzel/CompressionVAE..
+    It is based on the idea of Variational Autoencoders.
+    It encodes ``Document`` content from an ndarray in size `B x T` into an ndarray
+    in size `B x D`. Where `B` is the batch's size and `T` and `D`
+    are the dimensions pre (`T`) and after (`D`)the compression.
+
+    Full code and documentation can be found
+    `here <https://github.com/maxfrenzel/CompressionVAE>`_.
+
+    :param model_path: Path to the pretrained model
+    :param args:  Additional positional arguments
+    :param kwargs: Additional keyword arguments
     """
 
     def __init__(self, model_path: Optional[str] = 'model',
                  *args, **kwargs):
-        """
-        :param model_path: specifies the path to the pretrained model
-
-        """
         super().__init__(*args, **kwargs)
         self.model_path = model_path
 
     def post_init(self):
+        """Load VAE model"""
         super().post_init()
         from cvae import cvae
         import cvae.lib.model_iaf as model
@@ -78,13 +84,18 @@ class CompressionVaeEncoder(TFDevice, BaseNumericEncoder):
 
     @batching
     @as_ndarray
-    def encode(self, data: 'np.ndarray', *args, **kwargs) -> 'np.ndarray':
+    def encode(self, content: 'np.ndarray', *args, **kwargs) -> 'np.ndarray':
         """
-        :param data: a `B x T` numpy ``ndarray``, `B` is the size of the batch
-        :return: a `B x D` numpy ``ndarray``
+        Encode `Document` content from an ndarray in size `B x T` into an ndarray
+        in size `B x D`.
+
+        :param content: a `B x T` numpy ndarray
+        :return: a `B x D` numpy ndarray
+        :param args:  Additional positional arguments
+        :param kwargs: Additional keyword arguments
         """
         return self.sess.run([self.embeddings],
-                             feed_dict={self.data_feature_placeholder: data})[0]
+                             feed_dict={self.data_feature_placeholder: content})[0]
 
     def close(self) -> None:
         super().close()
