@@ -6,7 +6,7 @@ from jina.executors.encoders.numeric import TransformEncoder
 
 class IncrementalPCAEncoder(TransformEncoder):
     """
-    Encodes data from an ndarray in size `B x T` into an ndarray in size `B x D`.
+    Encodes ``Document`` content from an ndarray in size `B x T` into an ndarray in size `B x D`.
 
     Where `B` is the batch's size and `T` and `D` are the dimensions pre (`T`)
     and after (`D`) the encoding.
