@@ -33,13 +33,21 @@ def mixin_base_pod_parser(parser):
         '`port_in` and `port_out` will be set to random, '
         'and routers will be added automatically when necessary',
     )
+    gp.add_argument(
+        '--replicas',
+        type=int,
+        default=1,
+        help='The number of replicas in the pod, '
+        '`port_in` and `port_out` will be set to random, '
+        'and routers will be added automatically when necessary',
+    )
     gp.add_argument(
         '--polling',
         type=PollingType.from_string,
         choices=list(PollingType),
         default=PollingType.ANY,
         help='''
-The polling strategy of the Pod (when `parallel>1`) 
+The polling strategy of the Pod (when `parallel>1`)
 - ANY: only one (whoever is idle) Pea polls the message
 - ALL: all Peas poll the message (like a broadcast)
 ''',
@@ -64,12 +72,9 @@ The polling strategy of the Pod (when `parallel>1`)
     )
     gp.add_argument(
         '--peas-hosts',
-        action=KVAppendAction,
-        metavar='KEY: VALUE',
-        nargs='*',
-        help='''The hosts of the peas when parallel greater than 1,
-        pea have a new host address if the pea_id present in the map.
-        otherwise pea host will be identical to the host of pod.
-        Represented as a key value pair in argument.
-        key is the pea_id, and value is the host address.''',
+        nargs='+',
+        type=str,
+        help='''The hosts of the peas when parallel greater than 1.
+        Peas will be evenly distributed among the hosts. By default,
+        peas are running in the same host as the pod.''',
     )
