@@ -1,10 +1,13 @@
 import os
 
 from jina import Segmenter, Crafter
+from jina.executors.decorators import single
 
 
 class SimpleCrafter(Crafter):
     """Simple crafter for multimodal example."""
+
+    @single
     def craft(self, tags):
         """
         Read the data and add tags.
@@ -12,13 +15,16 @@ class SimpleCrafter(Crafter):
         :param tags: tags of data
         :return: crafted data
         """
-        return {'text': tags['caption'],
-                'uri': f'{os.environ["HW_WORKDIR"]}/people-img/{tags["image"]}'}
+        return {
+            'text': tags['caption'],
+            'uri': f'{os.environ["HW_WORKDIR"]}/people-img/{tags["image"]}',
+        }
 
 
 class BiSegmenter(Segmenter):
     """Segmenter for multimodal example."""
 
+    @single(slice_nargs=2)
     def segment(self, text, uri):
         """
         Segment data into text and uri.
@@ -29,5 +35,5 @@ class BiSegmenter(Segmenter):
         """
         return [
             {'text': text, 'mime_type': 'text/plain'},
-            {'uri': uri, 'mime_type': 'image/jpeg'}
+            {'uri': uri, 'mime_type': 'image/jpeg'},
         ]
