@@ -1,10 +1,11 @@
 from collections.abc import MutableSequence
-from typing import Union, Iterable, Tuple, Sequence, List
+from typing import Union, Iterable, Tuple, Sequence, List, Optional
 
 import numpy as np
 
 from ...helper import typename
 from ...logging import default_logger
+from ...enums import EmbeddingClsType
 
 try:
     # when protobuf using Cpp backend
@@ -20,10 +21,41 @@ except:
 from ...proto.jina_pb2 import DocumentProto
 from .traversable import TraversableSequence
 
+__all__ = ['DocumentSet']
+
 if False:
     from ..document import Document
+    from scipy.sparse import coo_matrix
+    from scipy.sparse import coo_matrix
 
-__all__ = ['DocumentSet']
+    # fix type-hint complain for sphinx and flake
+    from typing import TypeVar
+    import numpy as np
+    import scipy
+    import tensorflow as tf
+    import torch
+
+    EmbeddingType = TypeVar(
+        'EncodingType',
+        np.ndarray,
+        scipy.sparse.csr_matrix,
+        scipy.sparse.coo_matrix,
+        scipy.sparse.bsr_matrix,
+        scipy.sparse.csc_matrix,
+        torch.sparse_coo_tensor,
+        tf.SparseTensor,
+    )
+
+    SparseEmbeddingType = TypeVar(
+        'SparseEmbeddingType',
+        np.ndarray,
+        scipy.sparse.csr_matrix,
+        scipy.sparse.coo_matrix,
+        scipy.sparse.bsr_matrix,
+        scipy.sparse.csc_matrix,
+        torch.sparse_coo_tensor,
+        tf.SparseTensor,
+    )
 
 
 class DocumentSet(TraversableSequence, MutableSequence):
@@ -64,7 +96,7 @@ class DocumentSet(TraversableSequence, MutableSequence):
     def __len__(self):
         return len(self._docs_proto)
 
-    def __iter__(self):
+    def __iter__(self) -> 'Document':
         from ..document import Document
 
         for d in self._docs_proto:
@@ -160,7 +192,78 @@ class DocumentSet(TraversableSequence, MutableSequence):
                 and the documents have no embedding in a :class:`DocumentSet`.
         :rtype: A tuple of embedding in :class:`np.ndarray`
         """
-        return self.extract_docs('embedding')
+        return self.extract_docs('embedding', stack_contents=True)
+
+    def get_all_sparse_embeddings(
+        self, embedding_cls_type: EmbeddingClsType
+    ) -> Tuple['SparseEmbeddingType', 'DocumentSet']:
+        """Return all embeddings from every document in this set as a sparse array
+
+        :param embedding_cls_type: Type of sparse matrix backend, e.g. `scipy`, `torch` or `tf`.
+
+        :return: The corresponding documents in a :class:`DocumentSet`,
+            and the documents have no embedding in a :class:`DocumentSet`.
+        :rtype: A tuple of embedding and DocumentSet as sparse arrays
+        """
+
+        def stack_embeddings(embeddings):
+            if embedding_cls_type.is_scipy:
+                import scipy
+
+                return scipy.sparse.vstack(embeddings)
+            elif embedding_cls_type.is_torch:
+                import torch
+
+                return torch.vstack(embeddings)
+            elif embedding_cls_type.is_tf:
+                return embeddings
+            else:
+                raise ValueError(
+                    f'Trying to stack sparse embeddings with embedding_cls_type {embedding_cls_type} failed'
+                )
+
+        def get_sparse_ndarray_type_kwargs():
+            if embedding_cls_type.is_scipy:
+                from jina.types.ndarray.sparse.scipy import SparseNdArray
+
+                if not embedding_cls_type.is_scipy_stackable not in ['coo', 'csr']:
+                    default_logger.warning(
+                        f'found `{embedding_cls_type.name}` matrix, recommend to use `coo` or `csr` type.'
+                    )
+                return SparseNdArray, {'sp_format': embedding_cls_type.scipy_cls_type}
+            elif embedding_cls_type.is_torch:
+                from jina.types.ndarray.sparse.pytorch import SparseNdArray
+
+                return SparseNdArray, {}
+            elif embedding_cls_type.is_tf:
+                from jina.types.ndarray.sparse.tensorflow import SparseNdArray
+
+                return SparseNdArray, {}
+            else:
+                raise ValueError(
+                    f'Trying to get sparse embeddings with embedding_cls_type {embedding_cls_type} failed'
+                )
+
+        embeddings = []
+        docs_pts = []
+        bad_docs = []
+        sparse_ndarray_type, sparse_kwargs = get_sparse_ndarray_type_kwargs()
+        for doc in self:
+            embedding = doc.get_sparse_embedding(
+                sparse_ndarray_cls_type=sparse_ndarray_type, **sparse_kwargs
+            )
+            if embedding is None:
+                bad_docs.append(doc)
+                continue
+            embeddings.append(embedding)
+            docs_pts.append(doc)
+
+        if bad_docs:
+            default_logger.warning(
+                f'found {len(bad_docs)} docs at granularity {bad_docs[0].granularity} are missing sparse_embedding'
+            )
+
+        return stack_embeddings(embeddings), docs_pts
 
     @property
     def all_contents(self) -> Tuple['np.ndarray', 'DocumentSet']:
@@ -170,19 +273,21 @@ class DocumentSet(TraversableSequence, MutableSequence):
                 and the documents have no contents in a :class:`DocumentSet`.
         :rtype: A tuple of embedding in :class:`np.ndarray`
         """
-        return self.extract_docs('content')
+        # stack true for backward compatibility, but will not work if content is blob of different shapes
+        return self.extract_docs('content', stack_contents=True)
 
     def extract_docs(
-        self, *fields: str
+        self, *fields: str, stack_contents: Union[bool, List[bool]] = False
     ) -> Tuple[Union['np.ndarray', List['np.ndarray']], 'DocumentSet']:
         """Return in batches all the values of the fields
 
         :param fields: Variable length argument with the name of the fields to extract
+        :param stack_contents: boolean flag indicating if output lists should be stacked with `np.stack`
         :return: Returns an :class:`np.ndarray` or a list of :class:`np.ndarray` with the batches for these fields
         """
 
         list_of_contents_output = len(fields) > 1
-        contents = [[] for _ in fields if len(fields) > 1]
+        contents = [[] for _ in fields if list_of_contents_output]
         docs_pts = []
         bad_docs = []
 
@@ -195,8 +300,19 @@ class DocumentSet(TraversableSequence, MutableSequence):
                 for i, c in enumerate(content):
                     contents[i].append(c)
                 docs_pts.append(doc)
-            for i in range(len(contents)):
-                contents[i] = np.stack(contents[i])
+            for idx, c in enumerate(contents):
+                if not c:
+                    continue
+                if (
+                    isinstance(stack_contents, bool)
+                    and stack_contents
+                    and not isinstance(c[0], bytes)
+                ) or (
+                    isinstance(stack_contents, list)
+                    and stack_contents[idx]
+                    and not isinstance(c[0], bytes)
+                ):
+                    contents[idx] = np.stack(c)
         else:
             for doc in self:
                 content = doc.get_attrs_values(*fields)[0]
@@ -205,7 +321,19 @@ class DocumentSet(TraversableSequence, MutableSequence):
                     continue
                 contents.append(content)
                 docs_pts.append(doc)
-            contents = np.stack(contents) if contents else None
+
+            if not contents:
+                contents = None
+            elif (
+                isinstance(stack_contents, bool)
+                and stack_contents
+                and not isinstance(contents[0], bytes)
+            ) or (
+                isinstance(stack_contents, list)
+                and stack_contents[0]
+                and not isinstance(contents[0], bytes)
+            ):
+                contents = np.stack(contents)
 
         if bad_docs:
             default_logger.warning(
@@ -213,6 +341,9 @@ class DocumentSet(TraversableSequence, MutableSequence):
                 f'following fields: {fields} '
             )
 
+        if not docs_pts:
+            default_logger.warning('no documents are extracted')
+
         return contents, DocumentSet(docs_pts)
 
     def __bool__(self):
